from app.utils.logging import AutofixLogger
from managed_servers import Service, ManagementType, AutoFixStatus
from host_check import run_service_check
from app.config import logger, SERVER_MANAGEMENT_TYPE
from app.utils.check_processing import (
    process_script_status,
    process_auto_fix_results,
    process_report,
)


def process_service_check(s: Service):
    acknowledged = None
    # Skip  checks not applicable for Basic management
    if SERVER_MANAGEMENT_TYPE == ManagementType.BASIC.codename and s in [
        Service.BACKUP_DISK,
        Service.BACKUP_AGE,
    ]:
        logger.info("Service check not applicable - skipping")
        return

    # If a resource-intensive check has been successfully reported to API less than 24 hours ago, skip it
    if s in [Service.DISK, Service.BACKUP_DISK]:
        log_check_result = AutofixLogger.check_logged_api_reports(s.codename)
        if log_check_result.success:
            if log_check_result.data["already_sent"]:
                logger.info(
                    f"Service {s.codename} was in critical state less that 24 hours ago at {log_check_result.data['timestamp']} and has been reported. Skipping this check."
                )
                return
        else:
            logger.error(log_check_result.message)

    result = run_service_check(s)

    if not result.success:
        logger.error(
            f"Failed to run result check for service {s.codename}: {result.message}"
        )
        return

    if result.warnings:
        for warn in result.warnings:
            logger.warning(warn)

    data = result.data

    script_run_status = process_script_status(s.codename, data)
    if script_run_status.data.get("abort_execution", False):
        return

    auto_fix_results_check = process_auto_fix_results(s, data)
    if auto_fix_results_check.data["abort_execution"]:
        return
    data.update(auto_fix_results_check.data)

    if script_run_status.data.get("report_error", False):
        data["payload"]["error"] = script_run_status.data.get("error", "N/A")

    auto_fix_status = data.get("auto_fix_status")
    client_notified = data.get("client_notified")

    if not auto_fix_status:
        auto_fix_status = AutoFixStatus.PROCESS_MANUALLY.codename

    process_report(acknowledged, client_notified, data, result, s, auto_fix_status)


def main():
    for s in Service:
        process_service_check(s)


if __name__ == "__main__":
    main()
